#========================================================================================
# package      DE_EPAGES::AvaTax::t::unit::RemoteConnector
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  Unit test for the DE_EPAGES::AvaTax::API::RemoteConnector
#========================================================================================
package DE_EPAGES::AvaTax::t::unit::RemoteConnector;

use DE_EPAGES::Test::Mock::EpagesObject;
use DE_EPAGES::Test::Mock::Mockify;
use base DE_EPAGES::Core::API::Test;
use Test::More;

use strict;



sub test {
    my $self = shift;

    $self->test_RemoteTaxService();
    $self->test_RemoteTaxServiceInTestMode();
    $self->test_RemoteTaxServiceWithLogging();

    $self->test_RemoteAddressService();
    $self->test_RemoteAddressServiceInTestMode();
    $self->test_RemoteAddressServiceWithLogging();

    done_testing();
}



sub test_RemoteTaxService {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $IsShopInTestMode = 0;
    my $IsLoggingActivated = 0;
    my $Shop = $self->_createMockedShop($IsShopInTestMode);
    my $RemoteConnector = $self->_createRemoteConnector($Shop, $IsLoggingActivated);

    my $Remote = $RemoteConnector->connectToTaxService();

    is($Remote->{'ServiceURL'}, 'http://service.live.url', "$SubTestName: test if service url is correct.");
    is($Remote->{'Username'}, '12345', "$SubTestName: test if username is correct.");
    is($Remote->{'Password'}, '31415', "$SubTestName: test if password is correct");
    is($Remote->{'logging'}, '0', "$SubTestName: test if logging is deactivated.");
}



sub test_RemoteTaxServiceInTestMode {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $IsShopInTestMode = 1;
    my $IsLoggingActivated = 0;
    my $Shop = $self->_createMockedShop($IsShopInTestMode);
    my $RemoteConnector = $self->_createRemoteConnector($Shop, $IsLoggingActivated);

    my $Remote = $RemoteConnector->connectToTaxService();

    is($Remote->{'ServiceURL'}, 'http://service.test.url', "$SubTestName: test if service url is correct.");
    is($Remote->{'Username'}, '12345', "$SubTestName: test if username is correct.");
    is($Remote->{'Password'}, '31415', "$SubTestName: test if password is correct");
    is($Remote->{'logging'}, '0', "$SubTestName: test if logging is deactivated.");
}



sub test_RemoteTaxServiceWithLogging {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $IsShopInTestMode = 0;
    my $IsLoggingActivated = 1;
    my $Shop = $self->_createMockedShop($IsShopInTestMode);
    my $RemoteConnector = $self->_createRemoteConnector($Shop, $IsLoggingActivated);

    my $Remote = $RemoteConnector->connectToTaxService();

    is($Remote->{'ServiceURL'}, 'http://service.live.url', "$SubTestName: test if service url is correct.");
    is($Remote->{'Username'}, '12345', "$SubTestName: test if username is correct.");
    is($Remote->{'Password'}, '31415', "$SubTestName: test if password is correct");
    is($Remote->{'logging'}, '1', "$SubTestName: test if logging is deactivated.");
}



sub test_RemoteAddressService {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $IsShopInTestMode = 0;
    my $IsLoggingActivated = 0;
    my $Shop = $self->_createMockedShop($IsShopInTestMode);
    my $RemoteConnector = $self->_createRemoteConnector($Shop, $IsLoggingActivated);

    my $Remote = $RemoteConnector->connectToAddressService();

    is($Remote->{'ServiceURL'}, 'http://address.live.url', "$SubTestName: test if service url is correct.");
    is($Remote->{'Username'}, '12345', "$SubTestName: test if username is correct.");
    is($Remote->{'Password'}, '31415', "$SubTestName: test if password is correct");
    is($Remote->{'logging'}, '0', "$SubTestName: test if logging is deactivated.");
}



sub test_RemoteAddressServiceInTestMode {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $IsShopInTestMode = 1;
    my $IsLoggingActivated = 0;
    my $Shop = $self->_createMockedShop($IsShopInTestMode);
    my $RemoteConnector = $self->_createRemoteConnector($Shop, $IsLoggingActivated);

    my $Remote = $RemoteConnector->connectToAddressService();

    is($Remote->{'ServiceURL'}, 'http://address.test.url', "$SubTestName: test if service url is correct.");
    is($Remote->{'Username'}, '12345', "$SubTestName: test if username is correct.");
    is($Remote->{'Password'}, '31415', "$SubTestName: test if password is correct");
    is($Remote->{'logging'}, '0', "$SubTestName: test if logging is deactivated.");
}



sub test_RemoteAddressServiceWithLogging {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $IsShopInTestMode = 0;
    my $IsLoggingActivated = 1;
    my $Shop = $self->_createMockedShop($IsShopInTestMode);
    my $RemoteConnector = $self->_createRemoteConnector($Shop, $IsLoggingActivated);

    my $Remote = $RemoteConnector->connectToAddressService();

    is($Remote->{'ServiceURL'}, 'http://address.live.url', "$SubTestName: test if service url is correct.");
    is($Remote->{'Username'}, '12345', "$SubTestName: test if username is correct.");
    is($Remote->{'Password'}, '31415', "$SubTestName: test if password is correct");
    is($Remote->{'logging'}, '1', "$SubTestName: test if logging is deactivated.");
}



# ----------------------------------------------------------------



sub _createRemoteConnector {
    my $self = shift;
    my ($Shop, $IsLoggingActivated) = @_;

    my $MockifiedRemoteConnector = DE_EPAGES::Test::Mock::Mockify->new('DE_EPAGES::AvaTax::API::RemoteConnector', [$Shop]);
    $MockifiedRemoteConnector->addMockWithReturnValue('_LoadRootObject', $self->_createMockedSystem($IsLoggingActivated));

    return $MockifiedRemoteConnector->getMockObject();
}



sub _createMockedShop {
    my $self = shift;
    my ($IsInTestMode) = @_;

    my $hShopAttributes = {
        'TaxMethod' => $self->_createTaxMethod($IsInTestMode),
    };

    my $MockedShop = DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Shop::API::Object::Shop', $hShopAttributes);
    return $MockedShop;
}



sub _createTaxMethod {
    my $self = shift;
    my ($IsInTestMode) = @_;

    my $hTaxMethodAttributes = {
        'AvaTax_AccountNumber' => '12345',
        'AvaTax_LicenseKey' => '31415',
        'IsInTestMode' => $IsInTestMode,
        'instanceOf'    => 'TaxMethodAvaTax',
    };

    my $MockedTaxMethod = DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::AvaTax::API::Object::TaxMethodAvaTax', $hTaxMethodAttributes);
    return $MockedTaxMethod;
}



sub _createMockedSystem {
    my $self = shift;
    my ($IsLoggingActivated) = @_;

    my $hSystemAttributes = {
        'AvaTax_TaxServiceURL' => 'http://service.live.url',
        'AvaTax_TaxServiceURL_Test' => 'http://service.test.url',
        'AvaTax_AddressServiceURL' => 'http://address.live.url',
        'AvaTax_AddressServiceURL_Test' => 'http://address.test.url',
        'AvaTax_Logging' => $IsLoggingActivated,
    };

    my $MockedSystem = DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Object::API::Object::Site', $hSystemAttributes);
    return $MockedSystem;
}


__PACKAGE__->runScript();

1;
